package org.jutils.jdbc;

import oracle.jdbc.driver.OracleDriver;
import oracle.jdbc.driver.*;
import java.io.*;
import java.text.*;
import java.util.*;
/**
 * <p>------------------------------------------------------------------------------<br>
 * Name:     	OracleSqlFactory<br>
 * Project:   	jutils<br>
 * Copyright: 	jutils.org, see license file<br>
 * Comment:		Provides an easy-to-use- Singleton for creating SQL Query strings.<br>
 * Creation date: (07.07.2000 17:44:33)   <br>
 * Version:   <pre>$Id: OracleSqlFactory.java,v 1.40 2000/10/18 07:57:02 kkrafft2 Exp $</pre> <br>
 * ------------------------------------------------------------------------------<p>
 * @author: M.Winkler, doubleSlash GmbH
 */
import org.jutils.*;
import org.jutils.log.*;
import java.sql.*;
public class OracleSqlFactory implements SqlFactory, CharConstants {

    /** sql commands to be used. */
    public static final String QUERY_COMMAND = "SELECT";
    public static final String CREATE_TABLE_COMMAND = "CREATE TABLE";
    public static final String CREATE_SEQUENCE_COMMAND = "CREATE SEQUENCE";
    public static final String ALTER_COMMAND = "ALTER TABLE";
    public static final String UPDATE_COMMAND = "UPDATE";
    public static final String INSERT_COMMAND = "INSERT INTO";
    public static final String DROP_TABLE_COMMAND = "DROP TABLE";
    public static final String DROP_SEQUENCE_COMMAND = "DROP SEQUENCE";

    /** Some clauses needed to separate sql sections from each other. */
    public static final String FROM_CLAUSE = "FROM";
    public static final String WHERE_CLAUSE = "WHERE";
    public static final String FOR_CLAUSE = "FOR";
    public static final String ADD_CLAUSE = "ADD";
    public static final String DEFAULT_CLAUSE = "DEFAULT";
    public static final String STARTS_CLAUSE = "START WITH";
    public static final String INCREMENT_CLAUSE = "INCREMENT BY";
    public static final String VALUES_CLAUSE = "VALUES";
    public static final String SET_CLAUSE = "SET";
    public static final String PRIMARY_KEY_CLAUSE = "PRIMARY KEY";


    // The oracle specific type identifiers. To intended to be used outside this class. Therefore made private.
    private static final String ORACLE_TYPE_FOR_MEMO = "VARCHAR2(2000)";
    private static final String ORACLE_TYPE_FOR_STRING = "VARCHAR2(255)";
    private static final String ORACLE_TYPE_FOR_SHORT_STRING = "VARCHAR2(50)";
    private static final String ORACLE_TYPE_FOR_LONG = "NUMBER(20,0)";
    private static final String ORACLE_TYPE_FOR_DOUBLE = "NUMBER(10,10)";
    private static final String ORACLE_TYPE_FOR_FLOAT = "FLOAT";
    private static final String ORACLE_TYPE_FOR_DATE = "VARCHAR2(20)";
    private static final String ORACLE_TYPE_FOR_BLOB = "LONG RAW";
    private static final String ORACLE_TYPE_FOR_BOOLEAN = "NUMBER(1)";
    private static final String ORACLE_TYPE_FOR_PRIMARYKEY = ORACLE_TYPE_FOR_LONG;

    // some oracle specific constants
    private static final String ORACLE_SPECIFIC_TRUE_VALUE = "1";
    private static final String ORACLE_SPECIFIC_FALSE_VALUE = "0";
    private static final String ORACLE_SPECIFIC_STRING_IDENTIFIER = "'";

    /** The separator to be used to separate distinct columns. */
    public static final char COLUMN_SEPARATOR = ',';
    public static final char TABLE_SEPARATOR = ',';
    public static final char EXPRESSION_SEPARATOR = ',';

    /** Althougt this one really specifies a sentence terminator, it is not used to dispatch sql statements. */
    public static final String SENTENCE_TERMINATOR = ";";

    /** Defines wether or not line breaks are used. */
    public static final boolean USE_LINE_BREAKS = false;

    /** The name of the database product for which this factory can make statements. */
    public static final String DATABASE_PRODUCT_NAME = "Oracle";

    /** This one is used to "simulate" a blob where none is. */
    public final static byte[] ALIBI_BLOB = {0};

    /** Determines wether or not these many outputs happen. */
    private boolean debug_ = false;
    private DateFormat df_ = DateFormat.getInstance();
    private SimpleDateFormat sdf_ = (SimpleDateFormat) SimpleDateFormat.getInstance();



    /**
       * Creates a new Oracle sql factory.
       * <br>Creation date: (19.07.2000 16:13:59)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       */
    public OracleSqlFactory()
    {
    }



    /**
       * composeAddColumn method comment.
       */
    public StringBuffer composeAddColumn(java.lang.String tableName, String columnName, String fieldType, Object defaultValue, StringBuffer dest) throws ParseException
    {
        StringBuffer returnValue = dest;
        if (dest == null)
            dest = new StringBuffer();

        // append ALTER_COMMAND and a space
        dest.append(ALTER_COMMAND).append(SPACE);
        dest.append(tableName).append(SPACE);
        dest.append(ADD_CLAUSE).append(SPACE);

        // create declaration part
        dest.append(columnName).append(SPACE);
        dest.append(fieldType);

        // check wether a default value is present
        if (defaultValue != null)
        {
            String value = toDbValue(defaultValue, fieldType);
            dest.append(SPACE).append(DEFAULT_CLAUSE).append(SPACE);
            dest.append(toDbValue(defaultValue, fieldType));
        }
        return dest;
    }



    /**
     * composeNativeBlob method comment.
     */
    public java.sql.Blob composeBlob(Connection c, byte[] data) throws SQLException
    {
        OracleConnection nativeConnection = (OracleConnection) c;
        oracle.sql.BLOB returnValue = new oracle.sql.BLOB(nativeConnection);
        if (data != null)
            returnValue.setBytes(data);
        return (java.sql.Blob) returnValue;
    }



    /**
     * Composes a create sequence sql statement.
     * <br>Creation date: (27.07.2000 17:18:05)<br>
     * @author: M.Winkler, doubleSlash GmbH
     *
     * @return java.lang.StringBuffer The result.
     * @param name java.lang.String The name of the sequence.
     * @param initialValue long The initial value of the sequence.
     * @param increment long The distance between two values.
     * @param dest java.lang.StringBuffer Where the result should be put to. Will appended, not replaced.
     */
    public StringBuffer composeCreateSequence(String name, long initialValue, long increment, StringBuffer dest)
    {
        StringBuffer returnValue = dest;
        if (returnValue == null)
            returnValue = new StringBuffer();
        returnValue.append(CREATE_SEQUENCE_COMMAND).append(SPACE);
        returnValue.append(name).append(SPACE);
        returnValue.append(INCREMENT_CLAUSE).append(SPACE);
        returnValue.append(increment).append(SPACE);
        returnValue.append(STARTS_CLAUSE).append(SPACE);
        returnValue.append(initialValue);
        return returnValue;
    }



    /**
       * Creates a "CREATE TABLE"-Statement.
       * <br>Creation date: (14.07.2000 17:36:13)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String
       * @param tableName java.lang.String the name of the table to be created.
       * @param columns java.lang.String [] The names of the columns to be created.
       * @param types java.lang.String [] The types of the columns, in the same order like <code>columns</code>.
       * @param options java.lang.String [] Some product specific options that may be set.
       */
    public StringBuffer composeCreateTable(String tableName, String[] columns, String[] types, String[] defaultValues, StringBuffer destination) throws ParseException
    {
        // ---------------------------------------------------------------
        // Create return value and pass chreate table clause
        // ---------------------------------------------------------------
        StringBuffer returnValue = destination;
        String primaryKeyField = null;
        if (returnValue == null)
            returnValue = new StringBuffer();
        returnValue.append(CREATE_TABLE_COMMAND).append(SPACE);
        // if (!tableName.startsWith("'"))
        //   returnValue.append("'");
        returnValue.append(tableName);
        // if (!tableName.endsWith("'"))
        //   returnValue.append("'");
        returnValue.append(SPACE).append(OPEN_ROUND_BRACKET);

        // ---------------------------------------------------------------
        // go through both arrays and append them to the buffer
        // ---------------------------------------------------------------
        boolean somethingAdded = false;
        for (int index = 0; index < columns.length; index++)
        {
            if ((columns[index] != null) && (types[index] != null))
            {
                if (somethingAdded)
                {
                    // returnValue.append(SPACE).append(COLUMN_SEPARATOR).append(SPACE);
                    returnValue.append(COLUMN_SEPARATOR);
                }
                String name = columns[index];
                String type = pGetOracleType(types[index]);
                returnValue.append(name).append(SPACE).append(type);
                if (PRIMARY_KEY.equals(types[index]))
                {
                    if (primaryKeyField != null)
                    {
                        throw new ParseException("two primary keys have been passed with fields " + primaryKeyField + " AND " + name + ", which is not legal.", index);
                    }
                    else
                    {
                        primaryKeyField = name;
                        returnValue.append(SPACE).append(PRIMARY_KEY_CLAUSE);
                    }
                }

                // check for default values
                if ((defaultValues != null) && (defaultValues.length > index))
                {
                    String defaultValue = defaultValues[index];
                    if ((defaultValue != null) && (defaultValue != ""))
                    {
                        defaultValue = toDbValue(defaultValue, type);
                        returnValue.append(SPACE).append(DEFAULT_CLAUSE);
                        returnValue.append(SPACE).append(defaultValue);
                    }
                }
                somethingAdded = true;
            }
        }

        // ---------------------------------------------------------------
        // close statement and return as String
        // ---------------------------------------------------------------
        returnValue.append(CLOSE_ROUND_BRACKET);
        return returnValue;
    }



    /**
       * composeDropStatement method comment.
       */
    public java.lang.StringBuffer composeDropSequence(java.lang.String tableName, java.lang.StringBuffer dest)
    {
        // initialize
        StringBuffer returnValue = dest;
        if (returnValue == null)
            returnValue = new StringBuffer();

        // compose command
        returnValue.append(DROP_SEQUENCE_COMMAND).append(SPACE);
        returnValue.append(tableName);
        // returnValue.append(SENTENCE_TERMINATOR);

        // return result
        return returnValue;
    }



    /**
       * composeDropStatement method comment.
       */
    public java.lang.StringBuffer composeDropTable(java.lang.String tableName, java.lang.StringBuffer dest)
    {
        // initialize
        StringBuffer returnValue = dest;
        if (returnValue == null)
            returnValue = new StringBuffer();

        // compose command
        returnValue.append(DROP_TABLE_COMMAND).append(SPACE);
        returnValue.append(tableName);
        // returnValue.append(SENTENCE_TERMINATOR);

        // return result
        return returnValue;
    }



    /**
       * composeInsert method comment.
       */
    public java.lang.StringBuffer composeInsert(java.lang.String tableName, java.lang.String[] fields, java.lang.String[] types, java.lang.Object[] values, java.lang.StringBuffer destination)
    {
        // initialize
        StringBuffer returnValue = destination;
        StringBuffer valuesList = new StringBuffer();
        if (returnValue == null)
            returnValue = new StringBuffer();

        // compose sql command
        returnValue.append(INSERT_COMMAND).append(SPACE);
        returnValue.append(tableName).append(SPACE);
        returnValue.append(OPEN_ROUND_BRACKET);
        valuesList.append(VALUES_CLAUSE).append(SPACE).append(OPEN_ROUND_BRACKET);



        // scanning the arrays
        // ---------------------------------------------------------------

        // check fields and values arrays
        boolean canScan = (fields != null) && (values != null) && (fields.length > 0) && (values.length > 0);
        if (canScan)
        {
            int scannableSize = Math.min(Math.min(fields.length, values.length), types.length);
            boolean alreadyAppendedSomething = false;
            for (int index = 0; index < scannableSize; index++)
            {
                if (values[index] != null)
                {
                    try
                    {
                        String value = toDbValue(values[index], types[index]);
                        if (alreadyAppendedSomething)
                        {
                            returnValue.append(COLUMN_SEPARATOR).append(SPACE);
                            valuesList.append(COLUMN_SEPARATOR).append(SPACE);
                            alreadyAppendedSomething = true;
                        }
                        returnValue.append(fields[index]);
                        valuesList.append(value);
                    }
                    catch (ParseException pe)
                    {
                        String message = "Could not parse a field value to its original type :" + fields[index] + "=" + values[index];
                        System.err.println(message);
                    }
                }
                int skippedIndexes = Math.max(Math.max(fields.length, values.length), types.length) - scannableSize;
                if (skippedIndexes > 0)
                    LogManager.log("Lines ignored by OracleSqlFactory.composeInsert :" + skippedIndexes);
            }
            returnValue.append(CLOSE_ROUND_BRACKET).append(SPACE);
            valuesList.append(CLOSE_ROUND_BRACKET);
            returnValue.append(valuesList);
        }
        return returnValue;
    }



    /**
       * Composes a SQL Select statement, according to the provided values.
       * Creation date: (07.07.2000 17:49:46)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @param table java.lang.String [] The name of the tables to be accessed.
       * @param columns java.lang.String[] The names of the columns to be accessed.
       * @param expressions java.lang.String[]
       * @param modifiers java.lang.String [] a modifier (one of the constants).
       */
    public StringBuffer composeQuery(String[] tables, String[] columns, String[] expressions, StringBuffer dest)
    {

        // ---------------------------------------------------------------
        // initialize returnValue
        // ---------------------------------------------------------------
        StringBuffer returnValue = dest;
        if (returnValue == null)
            returnValue = new StringBuffer();
        else
            returnValue.setLength(0);

        // ---------------------------------------------------------------
        // create "SELECT "- section
        // ---------------------------------------------------------------
        returnValue.append(QUERY_COMMAND).append(SPACE);

        // insert column names (if present)
        boolean alreadyAppendedSomething = false;
        if ((columns != null) && (columns.length > 0))
        {
            for (int columnIndex = 0; columnIndex < columns.length; columnIndex++)
            {
                String column = columns[columnIndex];
                if (column != null)
                {
                    if (alreadyAppendedSomething)
                        returnValue.append(COLUMN_SEPARATOR);
                    returnValue.append(column);
                    alreadyAppendedSomething = true;
                }
            }
        }
        else
            returnValue.append(ASTERISK);

        // ---------------------------------------------------------------
        // create "FROM" - section
        // ---------------------------------------------------------------
        returnValue.append(SPACE);
        returnValue.append(FROM_CLAUSE);
        returnValue.append(SPACE);
        boolean somethingAdded = false;
        for (int tableIndex = 0; tableIndex < tables.length; tableIndex++)
        {
            if (tables[tableIndex] != null)
            {
                if (somethingAdded)
                    returnValue.append(TABLE_SEPARATOR);
                returnValue.append(tables[tableIndex]);
                somethingAdded = true;
            }
        }

        // ---------------------------------------------------------------
        // create "WHERE" - section
        // ---------------------------------------------------------------
        if ((expressions != null) && (expressions.length > 0))
        {
            returnValue.append(SPACE);
            returnValue.append(WHERE_CLAUSE);
            returnValue.append(SPACE);
            somethingAdded = false;
            for (int expressionIndex = 0; expressionIndex < expressions.length; expressionIndex++)
            {
                if (expressions[expressionIndex] != null)
                {
                    if (somethingAdded)
                        returnValue.append(EXPRESSION_SEPARATOR);
                    returnValue.append(expressions[expressionIndex]);
                    somethingAdded = true;
                }
            }
        }

        // append terminator ;
        // returnValue.append(SENTENCE_TERMINATOR);

        // give it back
        return returnValue;
    }



    /**
       * Creates a default update command containing the passed columns and new values.
       * <br>Creation date: (18.07.2000 11:28:01)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.StringBuffer
       * @param table java.lang.String
       * @param updates java.lang.String[]
       * @param dest java.lang.StringBuffer
       */
    public StringBuffer composeUpdate(String table, String[] fields, String[] types, Object[] newValues, StringBuffer dest)
    {

        // prepare
        StringBuffer returnValue = dest;
        if (returnValue == null)
            returnValue = new StringBuffer();
        String primaryKeyField = null;
        String primaryKeyValue = null;


        // compose sql command
        returnValue.append(UPDATE_COMMAND).append(SPACE);
        returnValue.append(table).append(SPACE);
        returnValue.append(SET_CLAUSE).append(SPACE);

        // compose columns
        boolean alreadyAppendedAColumn = false;
        int processablePositions = Math.min(Math.min(fields.length, types.length), newValues.length);
        for (int index = 0; index < processablePositions; index++)
        {
            if ((newValues[index] != null) && (fields[index] != null) && (types[index] != null))
            {
                if (!PRIMARY_KEY.equals(types[index]))
                {
                    if (newValues[index] != null)
                    {
                        try
                        {
                            String dbValue = toDbValue(newValues[index], types[index]);
                            if (alreadyAppendedAColumn)
                                returnValue.append(COLUMN_SEPARATOR);
                            returnValue.append(fields[index]).append(EQUALS).append(dbValue);
                            alreadyAppendedAColumn = true;
                        }
                        catch (ParseException pe)
                        {
                            String message = "Cound not parse passed generic type " + types[index] + " (found at position " + index + " of the passed types array) to a corresponding oracle type.";
                            if (isDebug())
                                System.err.println(message);
                            throw new IllegalArgumentException(message);
                        }
                    }
                }
                else
                {
                    if (primaryKeyField != null)
                    {
                        throw new IllegalStateException("primaryKey twice in field identifier array : " + primaryKeyField + " and " + fields[index] + "!");
                    }
                    else
                    {
                        if (newValues[index] != null)
                        {
                            primaryKeyField = fields[index];
                            primaryKeyValue = newValues[index].toString();
                        }
                    }
                }
            }
        }
        if (primaryKeyValue != null)
        {
            returnValue.append(SPACE).append(WHERE_CLAUSE).append(SPACE).append(primaryKeyField).append(EQUALS).append(primaryKeyValue);
        }
        return returnValue;
    }



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return boolean
     */
    public boolean isDebug()
    {
        return debug_;
    }



    /**
     * loadBytes method comment.
     */
    public byte[] loadBytes(java.lang.String fieldIdentifier, java.sql.ResultSet resultSet) throws java.sql.SQLException
    {
        byte[] returnValue = null;
        OracleResultSet nativeResultSet = (OracleResultSet) resultSet;
        oracle.sql.Datum datum = (oracle.sql.Datum) nativeResultSet.getOracleObject(fieldIdentifier);
        if (datum != null)
        {
            returnValue = datum.getBytes();
        }
        else
            throw new SQLException("field '" + fieldIdentifier + "' not available.");
        return returnValue;
    }



    /**
       * Converts the unified type identifiers defined in SqlFactory to the proprietary oracle type strings.
       * <br>Creation date: (20.07.2000 12:01:00)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.lang.String
       * @param typeIdentifier java.lang.String
       */
    private String pGetOracleType(String typeIdentifier)
    {
        String returnValue = null;
        if (typeIdentifier.equals(PRIMARY_KEY))
            returnValue = ORACLE_TYPE_FOR_PRIMARYKEY;
        else
            if (typeIdentifier.equals(LONG))
                returnValue = ORACLE_TYPE_FOR_LONG;
            else
                if (typeIdentifier.equals(DOUBLE))
                    returnValue = ORACLE_TYPE_FOR_DOUBLE;
                else
                    if (typeIdentifier.equals(FLOAT))
                        returnValue = ORACLE_TYPE_FOR_FLOAT;
                    else
                        if (typeIdentifier.equals(SHORT_STRING))
                            returnValue = ORACLE_TYPE_FOR_SHORT_STRING;
                        else
                            if (typeIdentifier.equals(BOOLEAN))
                                returnValue = ORACLE_TYPE_FOR_BOOLEAN;
                            else
                                if (typeIdentifier.equals(STRING))
                                    returnValue = ORACLE_TYPE_FOR_STRING;
                                else
                                    if (typeIdentifier.equals(BLOB))
                                        returnValue = ORACLE_TYPE_FOR_BLOB;
                                    else
                                        if (typeIdentifier.equals(MEMO))
                                            returnValue = ORACLE_TYPE_FOR_MEMO;
                                        else
                                            if (typeIdentifier.equals(DATE))
                                                returnValue = ORACLE_TYPE_FOR_DATE;
        return returnValue;
    }



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @param newDebug boolean
     */
    public void setDebug(boolean newDebug)
    {
        debug_ = newDebug;
    }



    /**
       * parseBoolean method comment.
       */
    public boolean toBooleanField(java.lang.String dbReturnedStringValue) throws java.text.ParseException
    {
        Integer integer = Integer.valueOf(dbReturnedStringValue);
        return integer.intValue() != 0;
    }



    /**
     * toDateField method comment.
     */
    public java.util.Date toDateField(java.lang.String fieldValue) throws ParseException
    {
        java.util.Date returnValue = null;
        fieldValue = toStringField(fieldValue);
        if ((fieldValue != null) && !fieldValue.equals(EMPTY_STRING))
        {
            returnValue = JdbcTools.toDateField(fieldValue);
        }
        return returnValue;
    }



    /**
       * toFieldValue method comment.
       */
    public java.lang.String toDbValue(double param)
    {
        String returnValue = String.valueOf(param);
        return returnValue;
    }



    /**
       * toFieldValue method comment.
       */
    public java.lang.String toDbValue(float param)
    {
        String returnValue = String.valueOf(param);
        return returnValue;
    }



    /**
       * toFieldValue method comment.
       */
    public java.lang.String toDbValue(int param)
    {
        String returnValue = String.valueOf(param);
        return returnValue;
    }



    /**
       * toFieldValue method comment.
       */
    public java.lang.String toDbValue(long param)
    {
        String returnValue = String.valueOf(param);
        return returnValue;
    }



    /**
       * toDbValue method comment.
       */
    public java.lang.String toDbValue(Object value, java.lang.String fieldType) throws ParseException
    {
        String returnValue = null;

        // check types
        if (fieldType.equals(BOOLEAN))
            returnValue = toDbValue(((Boolean) value).booleanValue());
        else
            if (fieldType.equals(PRIMARY_KEY) || fieldType.equals(LONG))
                returnValue = toDbValue(((Long) value).longValue());
            else
                if (fieldType.equals(SHORT_STRING) || fieldType.equals(STRING) || fieldType.equals(MEMO))
                    returnValue = toDbValue(value.toString());
                else
                    if (fieldType.equals(DOUBLE))
                        returnValue = toDbValue(((Double) value).doubleValue());
                    else
                        if (fieldType.equals(FLOAT))
                            returnValue = toDbValue(((Float) value).floatValue());
                        else
                            if (fieldType.equals(DATE))
                                returnValue = toDbValue((java.util.Date) value);
                            else
                                if (fieldType.equals(AS_IS))
                                    returnValue = value.toString();
        return returnValue;
    }



    /**
       * Converts a java specific string to a db string.
       */
    public java.lang.String toDbValue(java.lang.String value)
    {
        StringBuffer returnValue = new StringBuffer();
        returnValue.append(ORACLE_SPECIFIC_STRING_IDENTIFIER);

        // find internal string identifier
        // --- to do

        // append the concrete string
        if (value.length() == 0)
            returnValue.append("null");
        else
            returnValue.append(value);

        // append terminating string identifier.
        returnValue.append(ORACLE_SPECIFIC_STRING_IDENTIFIER);
        return returnValue.toString();
    }



    /**
       * toDbValue method comment.
       */
    public java.lang.String toDbValue(String value, java.lang.String fieldType) throws ParseException
    {
        String returnValue = null;

        // check types
        if (fieldType.equals(BOOLEAN))
            returnValue = toDbValue(Boolean.valueOf(value).booleanValue());
        else
            if (fieldType.equals(PRIMARY_KEY) || fieldType.equals(LONG))
                returnValue = toDbValue(Long.valueOf(value).longValue());
            else
                if (fieldType.equals(SHORT_STRING) || fieldType.equals(STRING) || fieldType.equals(MEMO))
                    returnValue = toDbValue(value);
                else
                    if (fieldType.equals(DOUBLE))
                        returnValue = toDbValue(Double.valueOf(value).doubleValue());
                    else
                        if (fieldType.equals(FLOAT))
                            returnValue = toDbValue(Float.valueOf(value).floatValue());
                        else
                            if (fieldType.equals(DATE))
                            {
                                java.util.Date parsedDate = null;
                                try
                                {
                                    parsedDate = DateFormat.getInstance().parse(value);
                                }
                                catch (ParseException pe)
                                {
                                    try
                                    {
                                        parsedDate = SimpleDateFormat.getInstance().parse(value);
                                    }
                                    catch (ParseException pe2)
                                    {
                                        throw pe2;
                                    }
                                }
                                returnValue = toDbValue(parsedDate);
                            }
        return returnValue;
    }



    /**
     * Converts the passed db value to the corresponding Jdbc Date type.
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return java.lang.String
     * @param date java.util.Date
     */
    public java.lang.String toDbValue(java.util.Date date)
    {
        String returnValue = null;
        returnValue = ORACLE_SPECIFIC_STRING_IDENTIFIER + JdbcTools.toJDBCDate(date) + ORACLE_SPECIFIC_STRING_IDENTIFIER;
        return returnValue;
    }



    /**
       * toFieldValue method comment.
       */
    public java.lang.String toDbValue(boolean param)
    {
        String returnValue = param ? ORACLE_SPECIFIC_TRUE_VALUE : ORACLE_SPECIFIC_FALSE_VALUE;
        return returnValue;
    }



    /**
       * toDoubleField method comment.
       */
    public double toDoubleField(java.lang.String fieldValue)
    {
        double returnValue = Double.valueOf(fieldValue).doubleValue();
        return returnValue;
    }



    /**
     * toFloatField method comment.
     */
    public float toFloatField(java.lang.String fieldValue)
    {
        float returnValue = Float.valueOf(fieldValue).floatValue();
        return returnValue;
    }



    /**
       * toLongField method comment.
       */
    public long toLongField(java.lang.String fieldValue)
    {
        long returnValue = Long.valueOf(fieldValue).longValue();
        return returnValue;
    }



    /**
       * toStringField method comment.
       */
    public java.lang.String toStringField(java.lang.String dbValue)
    {
        String returnValue = null;
        if (dbValue != null)
        {
            returnValue = dbValue;
            if (returnValue.startsWith(ORACLE_SPECIFIC_STRING_IDENTIFIER))
                returnValue = dbValue.substring(1);
            if (returnValue.endsWith(ORACLE_SPECIFIC_STRING_IDENTIFIER))
                returnValue = returnValue.substring(0, dbValue.length() - 1);
            if (returnValue.equalsIgnoreCase("null"))
                returnValue = "";
        }
        else
            returnValue = "";
        return returnValue;
    }
}
