package org.jutils.runtime;

import java.util.*;
/*
 * This class is used to create an easy-to-use interface to an idle
 * thread that is running on low priority.
 * This is NOT derived from Thread, since I wanted to avoid
 * any side effects.<br>
 * View the main method for usage.<br>
 * @author: M.Winkler, doubleSlash NetBusiness GmbH
 */
public class IdleJobHandler implements Runnable {
    private List jobs_;
    /** If true, the thread has been notified to shut down. */
    private boolean shutDown_ = false;

    /** The number of milliseconds the thread waits after the last idle job before relooping. */
    private int DEFAULT_SLEEP_TIME = 5000;
    private static IdleJobHandler instance_ = null;
    private Thread myThread_ = null;
    private int passes_;
    private int sleepTime_ = DEFAULT_SLEEP_TIME;
    private boolean running_ = false;



    /**
     * Creates a new IdleThread with an empty jobs stack.
     */
    private IdleJobHandler() {
        super();
        jobs_ = new Vector(5);
        passes_ = 0;
    }



    /**
     * Adds a new job to the job stack. This is a fail safe implementation.
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @param newJob java.lang.Runnable
     */
    public void addJob(Runnable newJob) {
        if ((newJob != null) && !jobs_.contains(newJob)) {
            jobs_.add(newJob);
        }
    }



    /**
     * Returns the number of passes the thread has done with doing jobs.
     * Passes that are done but did not execute a job are ignored.
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @return int The number of passes with a job to to, so far.
     */
    public int getPasses() {
        return passes_;
    }



    /**
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @return int
     */
    public int getSleepTime() {
        return sleepTime_;
    }



    /**
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @return java.lang.Thread
     */
    public java.lang.Thread getThread() {
        if (myThread_ == null) {
            myThread_ = new Thread(this);
            myThread_.setPriority(Thread.MIN_PRIORITY);
            myThread_.setName("IdleThread");
        }
        return myThread_;
    }



    /**
     * Returns the Singleton instance of the IdleThreadHandler.
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @return org.jutils.runtime.IdleThreadHandler
     */
    public static IdleJobHandler instance() {
        if (instance_ == null)
            instance_ = new IdleJobHandler();
        return instance_;
    }



    /**
     * Returns true if the thread still is inside the run() method.
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @return boolean
     */
    public boolean isRunning() {
        return running_;
    }



    /**
     * Joins the idle thread until its down.
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @exception java.lang.InterruptedException The exception description.
     */
    public void join() throws java.lang.InterruptedException {
        myThread_.join();
    }



    /**
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @param args java.lang.String[]
     */
    public static void main(String[] args) {
        System.out.println("Testing IdleJobHandler ...");
        System.out.println("Creating instance ...");
        IdleJobHandler handler = IdleJobHandler.instance();
        handler.start();
        handler.setSleepTime(500);
        System.out.println("Inserting an idle job ...");


        // creating a tester class ...
        class IdlePrintJob implements Runnable {
            public void run() {
                System.out.println("IdleRunner printing 'Hello' !");
            }
        }
        IdlePrintJob pJob = new IdlePrintJob();
        handler.addJob(pJob);
        System.out.println("Waiting for 5 passes ...");
        while (handler.getPasses() < 5) {
            System.out.println("... looping ...");
            try {
                synchronized (handler) {
                    handler.wait(2500);
                }
            }
            catch (InterruptedException ie) {
            }
        }
        System.out.println("shutting down idle job ...");
        handler.shutDown(true);
        System.out.println("Passes : " + handler.getPasses());
        System.out.println("Thats it!");
    }



    /**
     * Removes the passed job, if present. This is a fali safe implementation.
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @param jobToRemove java.lang.Runnable
     */
    public void removeJob(Runnable jobToRemove) {
        if ((jobToRemove != null) && (jobs_.contains(jobToRemove)))
            jobs_.remove(jobToRemove);
    }



    /**
     * Iterates through the job stack. CAUTION! DO NOT CALL MANUALLY!
     * This is the thread method, it definitifely DOES block the
     * calling block and therefore will cause a blocking system.
     * This method EXCLUSIVELY should be called by a thread executing its start() method.
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     */
    public void run() {
        running_ = true;
        Runnable job;
        while (!shutDown_) {
            synchronized (jobs_) {
                if (jobs_.size() > 0) {
                    passes_++;
                    for (int jobIndex = 0; jobIndex < jobs_.size(); jobIndex++) {
                        job = (Runnable) jobs_.get(jobIndex);
                        try {
                            job.run();
                        }
                        catch (Exception e) {
                            System.err.println("IdleJob " + job.toString() + " caused an uncaught exception:");
                            e.printStackTrace(System.err);
                        }
                    }
                }
            }
            try {
                // wait(SLEEP_TIME);
                Thread.currentThread().sleep(sleepTime_);
            }
            catch (InterruptedException ie) {
                // well,...
            }
        }
        //     System.out.println("Shutting down idle thread ...");
        //    Thread t = Thread.currentThread();

        // notifying all joined threads.
        synchronized (myThread_) {
            myThread_.notifyAll();
        }

        // clear ressources (if one wants to restart this one).
        myThread_ = null;
        running_ = false;
    }



    /**
     * Sets the time the thread sleeps before redoing the idle jobs.
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     * @param newSleepTime int The new value, in milliseconds.
     */
    public void setSleepTime(int newSleepTime) {
        sleepTime_ = newSleepTime;
    }



    /**
     * Sends a message to the run method to signal that
     * the surrounding systems are about to go down.
     * @param boolean waitUntilDone If true, the method joins the idle thread.
     * @return boolean True if the Thread has shutdown (which mostly is NOT the case when waitUntilDone==false).
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     *
     */
    public boolean shutDown(boolean waitUntilDone) {
        shutDown_ = true;
        if (myThread_ != null) {
            if (waitUntilDone && myThread_.isAlive()) {
                try {
                    myThread_.join();
                }
                catch (InterruptedException ie) {
                }
            }
        }
        return (myThread_ == null) || !myThread_.isAlive();
    }



    /**
     * Starts looping through the registered jobs.
     * @return boolean true if the Thread now is looping. Also returns thrue if the thread ALREADY is looping.
     * @author: M.Winkler, doubleSlash NetBusiness GmbH
     */
    public boolean start() {
        Thread t = getThread();
        if (!t.isAlive())
            t.start();
        return t.isAlive();
    }
}

