package org.jutils.text;

import java.util.*;
import org.jutils.collection.*;
/**
 * Implements an associator of string - string pairs.
 * The special thing about this is that you can search
 * for substring, and return the counterpart associated with
 * that substring.
 * @author: M.Winkler, doubleSlash GmbH
 */
public class PatternMatcher {

    // The minimal length of the partial key
    public static final int DEFAULT_FRAGMENT_SIZE = 3;
    private long maxId = 0;
    private int fragmentSize_ = DEFAULT_FRAGMENT_SIZE;

    // contains the complete key-value-pairs
    Vector pairs_ = new Vector();

    // Contains the single three letter fragments
    private Associator fragments_ = new Associator();
    private Object mutex_ = new Integer(5);



    /**
         * Creates a new pattern matcher with a default fragment size.
         */
    public PatternMatcher() {
        super();
    }



    /**
         * Creates a new pattern matcher with the defined fragment size, which must be > 1!
         */
    public PatternMatcher(int fragmentSize) {
        super();
        if (fragmentSize < 2)
            throw new IllegalArgumentException("fragmentSize MUST be greater 1!");
        fragmentSize_ = fragmentSize;
    }



    /**
     * Returns true if that key has a value
     * in the map. The key is considered to be a complete key.
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return boolean
     * @param partialKey java.lang.String The partial key to check for.
     */
    public boolean containsKey(String fullKey) {
        synchronized (mutex_) {
            List associations = fragments_.getAssociations(fullKey);
            return (associations != null) && (associations.size() > 0);
        }
    }



    /**
         * Code to perform when this object is garbage collected.
         *
         * Any exception thrown by a finalize method causes the finalization to
         * halt. But otherwise, it is ignored.
         */
    protected void finalize() throws Throwable {
        // Insert code to finalize the receiver here.
        // This implementation simply forwards the message to super.  You may replace or supplement this.
        super.finalize();
    }



    /**
         * Returns an array of all stored fragments that will lead to a subset of results.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return java.lang.String[]
         */
    public String[] getFragmentsArray() {
        String[] returnValue = null;
        Object[] keysArray = fragments_.getKeysArray();
        returnValue = new String[keysArray.length];
        System.arraycopy(keysArray, 0, returnValue, 0, returnValue.length);
        return returnValue;
    }



    /**
         * This is the number of chars a single fragment has. It also limits the
         * amount of characters usable to put or get an element.
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return int
         */
    public int getFragmentSize() {
        return fragmentSize_;
    }



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @param args java.lang.String[]
     */
    public static void main(String[] args) {
        System.out.println("Test PatternMatcher ...");
        int errorCount = 0;
        int fragmentSize = 5;
    
        // --------------------------------------------------------------
        long startTicks = System.currentTimeMillis();
        PatternMatcher matcher = new PatternMatcher(fragmentSize);
        System.out.println("test putting some strings into it ...");
        String[] testArray = {"hahaha", "nununu", "tatata", "hanuta", "hatanu", "nutaha", "nuhata", "tanuha", "tahanu"};
        for (int index = 0; index < testArray.length; index++) {
            matcher.put(testArray[index], testArray[index]);
        }
    
        // --------------------------------------------------------------
        System.out.println("test searching substrings ...");
        for (int index = 0; index < testArray.length; index++) {
            String wholeString = testArray[index];
            String subString = wholeString;
    
            // "normal" test
            if (wholeString.length() >= matcher.getFragmentSize())
                subString = wholeString.substring(1, fragmentSize + 1);
            List results = matcher.get(subString);
            if (!results.contains(wholeString)) {
                System.out.println("Content mismatch in normal sub string test !");
                System.out.println("whole/sub were " + wholeString + "/" + subString);
                System.out.println("Results are :" + results);
                errorCount++;
            }
            for (int innerIndex = 0; innerIndex < results.size(); innerIndex++) {
                Object o = results.get(innerIndex);
                String singleResult = (String) o;
                if (!(singleResult.indexOf(subString) > -1)) {
                    System.out.println("wrong result!");
                    errorCount++;
                }
            }
    
            // extreme test
            subString = wholeString.substring(0, matcher.getFragmentSize());
            results = matcher.get(subString);
            if (!results.contains(wholeString)) {
                System.out.println("Content mismatch in extreme test for word beginnings!");
                System.out.println("whole/sub were " + wholeString + "/" + subString);
                System.out.println("Results are :" + results);
                errorCount++;
            }
            for (int innerIndex = 0; innerIndex < results.size(); innerIndex++) {
                String singleResult = (String) results.get(innerIndex);
                if (!(singleResult.indexOf(subString) > -1)) {
                    System.out.println("wrong result!");
                    errorCount++;
                }
            }
    
            // multipler test
            subString = wholeString.substring(0, 2);
            subString = subString + subString;
            subString = subString + subString;
            results = matcher.get(subString);
            if (results.size() > 0) {
                if (!results.contains(wholeString)) {
                    System.out.println("test for multiplied strings did return values!");
                    System.out.println("whole/sub were " + wholeString + "/" + subString);
                    System.out.println("Results are :" + results);
                    errorCount++;
                }
            }
        }
        long stopTicks = System.currentTimeMillis();
        String[] fragments = matcher.getFragmentsArray();
        Arrays.sort(fragments, String.CASE_INSENSITIVE_ORDER);
        StringBuffer outputLine = CollectionHelper.toStringBuffer(fragments, "{", ",", "}", null);
        System.out.println("fragments are :" + outputLine.toString());
        System.out.println("--------------------------------------------------------------");
        System.out.println("Ticks used :" + (stopTicks - startTicks));
        System.out.println("Errors detected :" + errorCount);
        System.exit(errorCount);
    }



    /**
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return Vector
         * @param completeString java.lang.String
         */
    private Vector pConvertToFragmentVector(String completeString) {
        int fragmentCount = completeString.length() - fragmentSize_ + 1;
        Vector returnValue = new Vector(fragmentCount);
        for (int index = 0; index < fragmentCount; index++) {
            int endIndex = index + fragmentSize_;
            String subString = completeString.substring(index, endIndex);
            returnValue.add(subString);
        }
        CollectionHelper.removeDoubles(returnValue, true);
        return returnValue;
    }



    /**
     * Puts an element into that StringHashMap
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @param key java.lang.String A single String word, may contain blanks.
     * @param value java.lang.Object
     */
    public void put(String key, Object value) {
    
        // check params
        if (value == null)
            throw new IllegalArgumentException("cannot handle null pointers.");
        StringColumnizer columnizer = new StringColumnizer(key, " ");
        synchronized (mutex_) {
            while (columnizer.hasMoreTokens()) {
                String token = columnizer.nextToken().trim();
                if (token.length() < fragmentSize_)
                    throw new IllegalArgumentException("the single words of the key put into the PatternMatcher MUST be of length >= fragmentSize!");
                pPut(token, value);
            }
        }
    }



    /**
     * Returns a value, starting from that key.
     * This key may be a substring of the original
     * key, but must contain of at least three characters.
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return java.lang.Vector The list of values associated with that partial key.
     * @param partialKey java.lang.String A partial key,
     * containing of at least three letters.
     */
    public List get(String partialKey) {
        // check params
        if (partialKey == null)
            throw new IllegalArgumentException("cannot handle null pointers.");
        List returnValue = new Vector();
        StringColumnizer columnizer = new StringColumnizer(partialKey, " ");
        synchronized (mutex_) {
            while (columnizer.hasMoreTokens()) {
                String token = columnizer.nextToken().trim();
                if (token.length() < fragmentSize_)
                    throw new IllegalArgumentException("the single words of the key put into the PatternMatcher MUST be of length >= fragmentSize!");
                List partialResult = pGet(token);
                
                if ((partialResult != null) && (partialResult.size() > 0))
                    returnValue.addAll(partialResult);
            }
        }
        return returnValue; 
    }



    /**
     * Returns a value, starting from that key.
     * This key may be a substring of the original
     * key, but must contain of at least three characters.
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return java.lang.Vector The list of values associated with that partial key.
     * @param partialKey java.lang.String A partial key,
     * containing of at least three letters.
     */
    public List pGet(String partialKey) {
        if (partialKey == null) {
            throw new IllegalArgumentException("partial key MUST be different from null.");
        }
        if (partialKey.length() < fragmentSize_) {
            throw new IllegalArgumentException("partial key must have more characters than a fragment has.");
        }
        List foundPairs = null;
    
        // call private pair getter method
        foundPairs = pGetPairs(partialKey);
    
    
        // Retrieve the values
        List returnValue = new Vector();
        if ((foundPairs != null) && (foundPairs.size() > 0)) {
            for (int index = 0; index < foundPairs.size(); index++) {
                Object[] pair = (Object[]) foundPairs.get(index);
                returnValue.add(pair[1]);
            }
        }
        CollectionHelper.removeDoubles(returnValue);
        return returnValue;
    }



    /**
     * Returns a value, starting from that key.
     * This key may be a substring of the original
     * key, but must contain of at least three characters.
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return java.lang.Vector The list of values associated with that partial key.
     * @param partialKey java.lang.String A partial key,
     * containing of at least three letters.
     */
    private List pGetPairs(String partialKey) {
    
        // init
        List returnValue = new Vector();
    
    
        // find all fragments
        Vector fragmentVector = pConvertToFragmentVector(partialKey);
        List cut = fragments_.getAssociations(fragmentVector.firstElement()); // first fill with elements.
    
        boolean stopLoop = (cut == null) || (cut.size() < 1);
        int fragmentsCount = fragmentVector.size();
        for (int index = 0; index < fragmentsCount; index++) {
            List associations = fragments_.getAssociations(fragmentVector.get(index));
            if ((associations != null) && (associations.size() > 0)) {
                // create cut with existing hits
                cut = CollectionHelper.createCut(cut, associations, null);
            }
            else {
                // huh, no hit for a fragment? CANCEL!
                cut.clear();
                break;
            }
        }
    
        // did we find something?
        if ((cut != null) && (cut.size() > 0)) {
            List foundIndexes = new Vector(cut);
            CollectionHelper.removeDoubles(foundIndexes, true);
    
            // check validity of found results
            for (int index = 0; index < foundIndexes.size(); index++) {
                Integer foundIndex = (Integer) foundIndexes.get(index);
    
                // get the according pair
                Object[] pair = (Object[]) pairs_.elementAt(foundIndex.intValue());
    
                // check wether the key string is contained.
                String key = pair[0].toString();
                if (key.indexOf(partialKey) > -1)
                    returnValue.add(pair);
            }
        }
        return returnValue;
    }



    /**
     * Puts an element into that StringHashMap
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @param key java.lang.String A single STring word, not containing any blanks.
     * @param value java.lang.Object
     */
    public void pPut(String key, Object value) {
    
        // check params
        if (key.length() < fragmentSize_)
            throw new IllegalArgumentException("the key put into the PatternMatcher MUST be of length >= fragmentSize!");
    
        // put into pairs_ vector
        Object[] pair = {key, value};
        pairs_.add(pair);
        Integer pairIndex = new Integer(pairs_.size() - 1);
    
        // create fragments and put them into the fragment associator.
        Vector fragmentVector = pConvertToFragmentVector(key);
        for (int index = 0; index < fragmentVector.size(); index++) {
            fragments_.associate(fragmentVector.elementAt(index), pairIndex);
        }
    
        // put the whole key into the inherited hashtable
        if (key.length() > fragmentSize_)
            fragments_.associate(key, pairIndex);
    }
}

